@@ -1,15 +1,10 @@
 #ifndef SOURCE_TOOLS_H
 #define SOURCE_TOOLS_H
 
-#define SOURCE_TOOLS_ENABLE_DEBUG_LOGGING
-
-#include <vector>
-#include <string>
-
 #include <sourcetools/core/core.h>
 #include <sourcetools/collection/collection.h>
+#include <sourcetools/utf8/utf8.h>
 #include <sourcetools/cursor/cursor.h>
-#include <sourcetools/r/r.h>
 #include <sourcetools/read/read.h>
 #include <sourcetools/tokenization/tokenization.h>
 
